CREATE DATABASE planeramera;

USE planeramera;

CREATE TABLE tokens (
    id        INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
    userid    VARCHAR(40) NOT NULL,
    access    VARCHAR(128) NOT NULL,
    created   TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
    refresh   VARCHAR(128) NOT NULL,
    tokentype VARCHAR(32) NOT NULL,
    expiry    DATETIME NOT NULL,
    CONSTRAINT nodups UNIQUE(userid)
);

CREATE TABLE planning (
    id           INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
    courseid    INT NOT NULL,
    activated   TIMESTAMP NULL DEFAULT NULL,
    CONSTRAINT nodups UNIQUE(courseid)
);

CREATE TABLE instruction (
    id        INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
    moduleid  INT NOT NULL,
    instruction TEXT NULL,
    CONSTRAINT nodups UNIQUE(moduleid)
);

CREATE TABLE activity (
    id           INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
    moduleitemid INT NOT NULL,
    startdate    DATE,
    stopdate     DATE,
    estimated    DECIMAL(5,1),
    updated      TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
    visible      BOOL NOT NULL DEFAULT FALSE,
    courseid     INT NOT NULL,
    CONSTRAINT nodups UNIQUE(moduleitemid)
);

CREATE TABLE activity_user (
    id           INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
    moduleitemid    INT NOT NULL,
    userid          INT NOT NULL,
    estimated       DECIMAL(5,1),
    imported        TIMESTAMP NULL DEFAULT NULL,
    CONSTRAINT nodups UNIQUE(moduleitemid, userid)
);


CREATE TABLE category (
    id      INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
    name    VARCHAR(128) NOT NULL
);

INSERT INTO category (name)
VALUES
    ("Read text, Swedish"),
    ("Read text, English"),
    ("Write text, Swedish"),
    ("Write text, English"),
    ("Calculations"),
    ("Practical work"),
    ("Group work"),
    ("Experience recording");

CREATE TABLE activity_category (
    id              INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
    activityid      INT NOT NULL,
    categoryid      INT NOT NULL,
    CONSTRAINT FOREIGN KEY (activityid) REFERENCES activity(id),
    CONSTRAINT FOREIGN KEY (categoryid) REFERENCES category(id),
    CONSTRAINT nodups UNIQUE(activityid, categoryid)
);

