# PlaneraMera

PlaneraMera is a tool for study planning that adds functionality to module functionality in Canvas and gives students the opportunity to import planning into their Canvas calendar.

## Starting PlaneraMera

PlaneraMera is registered as an external app in Canvas. The LTI configuration is located in `configs/lti.xml` and describes how PlaneraMera should be displayed and enabled in the interface of Canvas.

PlaneraMera is available as a menu item to add to course navigation by instructor of course.

### LTI launch

When a student or instructor clicks on the menu item, Canvas initiates startup by sending a POST request to the path `/lti/launch` (defined in `lti.xml: course_navigation.url`). A large number of parameters are included in request and the application uses the following:

* oauth_consumer_key
* oauth_nonce
* oauth_timestamp
* oauth_signature
* lti_message_type
* lti_version
* custom_canvas_user_id
* custom_canvas_course_id
* custom_canvas_api_domain
* user_id
* roles


PlaneraMera verifies that OAuth signature and values ​​are correct. The values ​​for `oauth.consumer_key` and` oauth.consumer_secret` in the configuration file `planeramera.json` must be exactly the same as the corresponding values ​​when registering the LTI app in Canvas (_Consumer key_ and _Shared key_).


PlaneraMera creates a tokenUserID unique to the Canvas environment + user. It is used as primary key in the `tokens` table for storing OAuth2 tokens.


PlaneraMera creates/gets a session unique to the Canvas environment + user + course. Stored in session:
* tokenUserID (md5-hash)
* customCanvasUserID (integer)
* customCanvasCourseID (integer)

PlaneraMera attempts to load the previous OAuth2 token from the table `tokens` in the database. If a token exists, it means that the user has approved the PlaneraMera app in Canvas. This only needs to be done once (per Canvas environment and users).


If no OAuth2 token found:
* User is redirected back to Canvas to approve app. Upon approval, the user is redirected to `/lti/checkOAuth2` (url defined in `planeramera.json: canvas.redirect_url`).
* The method `CheckOAuth2` checks that an authorization code and a session id are included in the request and exchanges authorization code for an access token and a refresh token. These are stored in the `tokens` table in the database and also in the session. The user is then redirected to `/?session=<sessionid>`.

If OAuth2 token found:
* user is redirected to `/?session=<sessionID>`

### Frontend: Starting angular app

The angular app is started from `web/static/index.html` where attribute `sessionToken` on `app-root` is set to value of the query parameter `session`.

The app component (in `app.component.ts`) reads value of `sessionToken` and sets `PMSessionToken` (in `app.module.ts`).

The service that handles API requests to the backend (in `backend.service.ts`) subscribes to changes to `PMSessionToken`. Upon change, the `Authorization` header is updated with the value `PM <sessionid>`.


### Backend: API request handling

Backend middleware  `SessionMW` (in `api.go`) is run on each incoming request against `/api/`. It checks that request has a valid `Authorization` header and uses session id (string after `PM`) to get the session from session storage. It gets `userID` and `courseID` from session and puts it in the context together with `token` and `sessionID`.

An API request without valid `Authorization` header and session id will return `401 Unauthorized`.

