package canvas

import (
	"bytes"
	"fmt"
	"mime/multipart"
	"net/http"
	"time"
)

// CalendarEventPost Creates a new event in users calendar
func (c Client) CalendarEventPost(moduleItemID int, userID int, startat string, endat string, title string, description string) (string, error) {

	// Parse the date string into Go's time object
	// The 1st param specifies the format, 2nd is our date string
	layout := "2006-01-02"
	myStartDate, err := time.Parse(layout, startat[0:10])
	if err != nil {
		fmt.Println(err)
	}
	myEndDate, err := time.Parse(layout, endat[0:10])
	if err != nil {
		fmt.Println(err)
	}
	// how many days are there in between those dates, add one so last day will get event as well
	numberOfDays := daysBetween(myStartDate, myEndDate)
	numberOfDays = numberOfDays + 1
	// create numberOfDays events
	for i := 0; i < numberOfDays; i++ {
		url := fmt.Sprintf("%s/calendar_events?", c.baseurl)
		method := "POST"
		payload := &bytes.Buffer{}
		writer := multipart.NewWriter(payload)
		err = writer.Close()
		if err != nil {
			fmt.Println(err)
		}
		req, err := http.NewRequest(method, url, payload)
		if err != nil {
			fmt.Println(err)
		}
		req.Header.Add("Content-Type", "application/json")
		req.Header.Set("Content-Type", writer.FormDataContentType())

		q := req.URL.Query()

		q.Add("calendar_event[context_code]", fmt.Sprintf("user_%d", userID))
		q.Add("calendar_event[title]", title)
		//loopdate should increase by one each iteration and convert back to string
		q.Add("calendar_event[start_at]", (myStartDate.AddDate(0, 0, i)).Format(layout))
		q.Add("calendar_event[end_at]", (myStartDate.AddDate(0, 0, i)).Format(layout))
		q.Add("calendar_event[all_day]", "true")
		q.Add("calendar_event[description]", description)

		req.URL.RawQuery = q.Encode()
		//fmt.Println(req.URL.String())
		res, err := c.client.Do(req)
		if err != nil {
			fmt.Println(err)
		}
		//fmt.Println(res.StatusCode)
		if res.StatusCode < 200 || res.StatusCode > 299 {
			return "ERR", fmt.Errorf("Canvas.Modules: Canvas said %v", res.Status)
		}

		defer res.Body.Close()
		// body, err := ioutil.ReadAll(res.Body)
		// fmt.Println(string(body))
	}
	return "OK", nil
}

func daysBetween(a, b time.Time) int {
	if a.After(b) {
		a, b = b, a
	}

	days := -a.YearDay()
	for year := a.Year(); year < b.Year(); year++ {
		days += time.Date(year, time.December, 31, 0, 0, 0, 0, time.UTC).YearDay()
	}
	days += b.YearDay()

	return days
}
