package planeramera

import (
	"encoding/json"
	"os"
)

// Config represents a configuration
type Config struct {
	OAuth struct {
		ConsumerKey    string `json:"consumer_key"`
		ConsumerSecret string `json:"consumer_secret"`
	} `json:"oauth"`
	HTTPS struct {
		CertPath string `json:"cert_path"`
		KeyPath  string `json:"key_path"`
	} `json:"https"`
	Canvas struct {
		BaseURL      string `json:"base_url"`
		ClientID     string `json:"client_id"`
		ClientSecret string `json:"client_secret"`
		AuthURL      string `json:"auth_url"`
		TokenURL     string `json:"token_url"`
		RedirectURL  string `json:"redirect_url"`
	} `json:"canvas"`
	DB struct {
		URN string `json:"urn"`
	} `json:"db"`
	HOSTPORT struct {
		PORT string `json:"port"`
	} `json:"hostport"`
}

// ReadConfig returns a config
func ReadConfig(path string) (Config, error) {
	var conf Config

	f, err := os.Open(path)
	if err != nil {
		return conf, err
	}
	defer f.Close()

	decoder := json.NewDecoder(f)
	if err = decoder.Decode(&conf); err != nil {
		return conf, err
	}

	return conf, nil
}
